==> galaxyhack-1.55.2-src.tar.bz2: src/RTS.cpp <==
/*
   Copyright (C) 2004 by James Gregory
   Part of the GalaxyHack project
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License.
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY.
 
   See the COPYING file for more details.
*/

#include "RTS.h"
#include "Globals.h"
#include "Group.h"
#include "PreBattle.h"
#include "TerrainTile.h"
#include "Projectile.h"

#include <vector>
#include <list>

using std::vector;
using std::list;

namespace RTS {

RTS_State::RTS_State() {
	radarDragging = false;
	lastAITime = SDL_GetTicks();
	lastScrollTime = SDL_GetTicks();
}

RTS_State::~RTS_State() {
	projectiles.clear();

	if (gsTo != GST_PreBattle && gsTo != GST_Score)
		PreBattle::Unload();
	else if (gsTo == GST_PreBattle)
		RestartPreBattle();
}

void RTS_State::Main() {
	ScrollAndDrag();

	if (now - lastAITime > worldUpdateInterval && !(paused)) {
		RunGroupAI();
		RunMoveCommands();
		RunFireCommands();
		Upkeep();

		lastAITime = SDL_GetTicks();

		++frameCounter;
	}
	
	if (skipDisplayFrame || globalSettings.batch)
		return;
		
	DrawWorld();
	DrawAllWindows();
}

void RTS_State::MouseD(Uint8 button, Uint16 x, Uint16 y) {
	RTSMouseD(button, x, y);
}

void RTS_State::MouseU(Uint8 button, Uint16 x, Uint16 y) {
	RTSMouseU(button, x, y);
}

void RTS_State::MouseM(Uint8 state, Uint16 x, Uint16 y) {
	RTSMouseM(state, x, y);
}

void RTS_State::Keyboard(SDL_keysym& keysym) {
	RTSKeyboard(keysym);
}

void RTSMouseD(Uint8 button, Uint16 x, Uint16 y) {
	if (button == SDL_BUTTON_RIGHT)
		myWindows.push_back(GenWindow(x, y, RTS_BasePU, 0, 0, 0));
	
	//radar before groups
	if (button == SDL_BUTTON_LEFT
	&& x > radarRect.x
	&& x < radarRect.x + radarRect.w
	&& y > radarRect.y
	&& y < radarRect.y + radarRect.h) {
		radarDragging = true;
		RTSMouseM(0, x, y);
	} else {
		for (int i = 0; i != sides.size(); ++i) {
			for (int j = 0; j != sides[i].groups.size(); ++j)
				sides[i].groups[j].MouseD(button, x, y);
		}
	}
}

void RTSMouseU(Uint8 button, Uint16 x, Uint16 y) {
	if (button == SDL_BUTTON_LEFT)
		radarDragging = false;

	for (int i = 0; i != sides.size(); ++i) {
		for (int j = 0; j != sides[i].groups.size(); ++j)
			sides[i].groups[j].MouseU(button, x, y);
	}
}

void RTSMouseM(Uint8 state, Uint16 x, Uint16 y) {
	//radar
	if (radarDragging
		&& x > radarRect.x
		&& x < radarRect.x + radarRect.w
		&& y > radarRect.y
		&& y < radarRect.y + radarRect.h) {
			int tempCenterx = (x - radarRect.x) * worldWidth / radarRect.w;
			int tempCentery = (y - radarRect.y) * worldHeight / radarRect.h;

			viewx = tempCenterx - (globalSettings.screenWidth >> 1);
			viewy = tempCentery - (globalSettings.screenHeight >> 1);

			viewSide = -1;
	}
}

void RTSKeyboard(SDL_keysym& keysym) {
	switch(keysym.sym) {
	case SDLK_ESCAPE:
		if (gsCurrent == GST_PreBattle && PreBattle::pbState == PBS_Position)
			gsTo = GST_ForceSelect;
		else
			gsTo = GST_MainMenu;
		return;
		break;

	case SDLK_F1:
		MessageWindows(RTS_FollowView, 0, 0, all, none);
		break;

	case SDLK_F2:
		for (int i = 0; i != sides.size(); ++i) {
			for (int j = 0; j != sides[i].groups.size(); ++j)
				sides[i].groups[j].ToggleDrawNumber();
		}
		break;

	case SDLK_F3:
		for (int i = 0; i != sides.size(); ++i) {
			for (int j = 0; j != sides[i].groups.size(); ++j)
				sides[i].groups[j].ToggleDrawBound();
		}
		break;

	case SDLK_F11:
		myWindows.push_back(GenWindow(0, 0, RTS_RestartQ, 0, 0, 0));
		break;
	
	case SDLK_SPACE: {
		static bool speedSlider = false;
		static int sliderWinID = 0;		
		if (speedSlider) {
			try {
				Slider* pSlider = dynamic_cast<Slider*>(LocateWindow(sliderWinID));
				pSlider->WinMessage(WC_YouClose, 0, 0, sliderWinID, none);
				speedSlider = false;
				break;
			} catch (runtime_error e) {
			}
		}
		//if we fell through due to a runtime error because window was already closed, create a new one
		myWindows.push_back(GenWindow(topRightBoxRect.x, topRightBoxRect.y + topRightBoxRect.h, RTS_SetGameSpeed, 0, 0, 0));
		sliderWinID = windowIDs;
		speedSlider = true;
		}
		break;
	
	case SDLK_KP_PLUS:
		SetWorldUpdateInterval(worldUpdateInterval - 10);
		break;
		
	case SDLK_KP_MINUS:
		SetWorldUpdateInterval(worldUpdateInterval + 10);
		break;
	
	case SDLK_p:
		if (paused == false)
			paused = true;
		else if (worldUpdateInterval < maxWorldUpdateInterval)
			paused = false;
		break;
		
	case SDLK_i:
		SetWorldUpdateInterval(standardInterval);
		break;
		
	case SDLK_o:
		SetWorldUpdateInterval(0);
		break;
	}
}

void ScrollAndDrag() {
	int state, x, y;
	state = SDL_GetMouseState(&x, &y);

	//scrolling via mouse, also used to break out of follow view
	if (now - lastScrollTime > scrollInterval) {
		if (x > globalSettings.screenWidth - 8) {
			viewx += screenMoveSpeed;
			viewSide = -1;
		}
		if (x < 8) {
			viewx -= screenMoveSpeed;
			viewSide = -1;
		}
		if (y > globalSettings.screenHeight - 8) {
			viewy += screenMoveSpeed;
			viewSide = -1;
		}
		if (y < 8) {
			viewy -= screenMoveSpeed;
			viewSide = -1;
		}

		lastScrollTime = SDL_GetTicks();
	}

	if (gsCurrent == GST_PreBattle) {
		for (int i = 0; i != sides.size(); ++i) {
			for (int j = 0; j != sides[i].groups.size(); ++j)
				sides[i].groups[j].Drag(state, x, y);
		}
	}
}

void RunGroupAI() {
	for (int i  = 0; i != sides.size(); ++i) {
		for (int j = 0; j != sides[i].groups.size(); ++j)
			sides[i].groups[j].RunGroupAI();
	}
}

void RunMoveCommands() {
	for (int i  = 0; i != sides.size(); ++i) {
		for (int j = 0; j != sides[i].groups.size(); ++j)
			sides[i].groups[j].Move();
	}

	for (list<Projectile>::iterator iter = projectiles.begin(); iter != projectiles.end();) {
		if (!iter->Move())
			iter = projectiles.erase(iter);
		else
			++iter;
	}
}

void RunFireCommands() {
	for (int i  = 0; i != sides.size(); ++i) {
		for (int j = 0; j != sides[i].groups.size(); ++j)
			sides[i].groups[j].RunFireCommands();
	}
}

void Upkeep() {
	for (int i  = 0; i != sides.size(); ++i) {
		for (int j = 0; j != sides[i].groups.size(); ++j)
			sides[i].groups[j].Upkeep();
	}

	int sidesLeft = 0;

	for (int i  = 0; i != sides.size(); ++i) {
		int sideTotalCS = sides[i].GetTotalCapShips();
		if (sideTotalCS)
			++sidesLeft;
	}

	if (sidesLeft < 2)
		gsTo = GST_Score;
	else if (globalSettings.batch && frameCounter > globalSettings.maxFrames)
		gsTo = GST_Score;
}

void DrawWorld() {
	FollowViewCenter();
	CheckViewPos();

	JSDL.BltFill(screenRect, 0);

	for (int i = 0; i != terrainTiles.size(); ++i)
		terrainTiles[i].DrawSelf();

	for (int i = 0; i != sides.size(); ++i) {
		for (int j = 0; j != sides[i].groups.size(); ++j)
			sides[i].groups[j].SetUSRect();
	}

	for (int i = 0; i != sides.size(); ++i) {
		for (int j = 0; j != sides[i].groups.size(); ++j)
			sides[i].groups[j].DrawSelfBackBack();
	}

	for (int i = 0; i != sides.size(); ++i) {
		for (int j = 0; j != sides[i].groups.size(); ++j)
			sides[i].groups[j].DrawSelfBack();
	}

	for (int i = 0; i != sides.size(); ++i) {
		for (int j = 0; j != sides[i].groups.size(); ++j)
			sides[i].groups[j].DrawSelfBack();
	}

	for (int i = 0; i != sides.size(); ++i) {
		for (int j = 0; j != sides[i].groups.size(); ++j)
			sides[i].groups[j].DrawSelfMiddle();
	}

	for (int i = 0; i != sides.size(); ++i) {
		for (int j = 0; j != sides[i].groups.size(); ++j)
			sides[i].groups[j].DrawSelfFront();
	}

	JSDL.LockBack();
	for (list<Projectile>::iterator iter = projectiles.begin(); iter != projectiles.end(); ++iter)
		iter->DrawSelfPixels();

	for (int i = 0; i != sides.size(); ++i) {
		for (int j = 0; j != sides[i].groups.size(); ++j)
			sides[i].groups[j].DrawSelfPixels();
	}
	JSDL.UnlockBack();

	for (list<Projectile>::iterator iter = projectiles.begin(); iter != projectiles.end(); ++iter)
		iter->DrawSelfBitmap();

	if (gsCurrent == GST_PreBattle) {
		for (int i = 0; i != sides.size(); ++i) {
			SDL_Rect tempRect;
			tempRect.x = sides[i].startingRect.x - viewx;
			tempRect.y = sides[i].startingRect.y - viewy;
			tempRect.w = sides[i].startingRect.w;
			tempRect.h = sides[i].startingRect.h;
			DrawRectBorder(tempRect, sides[i].color);
		}
	}

	DrawRadar();
}

void DrawRadar() {
	JSDL.BltFill(topRightBoxRect, gold);

	SDL_FillRect(radarSurface, 0, black);

	//Uint16 strided lpitch
	static const Uint16 lPitch = radarSurface->pitch >> 1;

	SDL_LockSurface(radarSurface);

	Uint16* videoBuffer = reinterpret_cast<Uint16*>(radarSurface->pixels);

	for (int i  = 0; i != sides.size(); ++i) {
		for (int j = 0; j != sides[i].groups.size(); ++j) {
			if (sides[i].groups[j].GetAlive()) {
				CoordsInt center = sides[i].groups[j].GetCenter();

				int radarx = center.x * radarRect.w / worldWidth;
				int radary = center.y * radarRect.h / worldHeight;

				videoBuffer[radary*lPitch + radarx] = sides[i].radarColor;
			}
		}
	}
	
	int radarx = viewx * radarRect.w / worldWidth;
	int radary = viewy * radarRect.h / worldHeight;
	
	static const int lineWidth = radarRect.w * globalSettings.screenWidth / worldWidth;
	static const int lineHeight = radarRect.h * globalSettings.screenHeight / worldHeight;
	
	for (int x = radarx; x != radarx + lineWidth + 1; ++x)
		videoBuffer[radary*lPitch + x] = white;
		
	for (int x = radarx; x != radarx + lineWidth + 1; ++x)
		videoBuffer[(radary + lineHeight)*lPitch + x] = white;
		
	for (int y = radary; y != radary + lineHeight + 1; ++y)
		videoBuffer[y*lPitch + radarx] = white;
		
	for (int y = radary; y != radary + lineHeight + 1; ++y)
		videoBuffer[y*lPitch + radarx + lineWidth] = white;

	SDL_UnlockSurface(radarSurface);

	JSDL.Blt(radarSurface, radarRect);
}

void SaveGame() {
    /* this should be staged so as to happen at the very end of a game loop, so we don't
    save some stuff out of sync with other stuff
    
    combine first 5 letters of each fleet fighting, divided by _s, to make a filename.
    if such a file already exists in saves directory, add a number on the end to make it unique
    
    Save all global variables - including static members, including random number "next"
    
    Save all variables for each side, and each group on each side, and each unit in each group
    */
}

void SetWorldUpdateInterval(int newValue) {
	worldUpdateInterval = newValue;
	
	if (worldUpdateInterval < 0)
		worldUpdateInterval = 0;
		
	if (worldUpdateInterval == maxWorldUpdateInterval)
		paused = true;
	else
		paused = false;
		
	if (worldUpdateInterval > maxWorldUpdateInterval)
		worldUpdateInterval = maxWorldUpdateInterval;

	if (worldUpdateInterval > standardInterval - 10 && worldUpdateInterval < standardInterval + 10)
		worldUpdateInterval = standardInterval;
}

//////

string MoveComToString(const AICommands& theCommands) {
	string ret;
	char output[30];

	if (!theCommands.bInverse)
		ret = "Moving towards ";
	else
		ret = "Moving away from ";

	switch (theCommands.moveCommand) {
	case MC_NoMove:
		ret = "Not moving";
		break;

	case MC_MoveCompass:
		switch (theCommands.moveTarget.y) {
		case 0:
			ret += "North";
			break;

		case 1:
			ret += "North-East";
			break;

		case 2:
			ret += "East";
			break;

		case 3:
			ret += "South-East";
			break;

		case 4:
			ret += "South";
			break;

		case 5:
			ret += "South-West";
			break;

		case 6:
			ret += "West";
			break;

		case 7:
			ret += "North-West";
			break;
		}
		break;

	case MC_MoveGroup:
		//+1 so side 1/group is 1
		sprintf(output, "%s Group %d", sides[theCommands.moveTarget.x].name.c_str(), theCommands.moveTarget.y + 1);

		ret += output;
		break;

	case MC_Patrol:
		ret = "Patrolling ";
		sprintf(output, "%s Group %d", sides[theCommands.moveTarget.x].name.c_str(), theCommands.moveTarget.y + 1);

		ret += output;
		break;
	}

	return ret;
}

string FireComToString(const AICommands& theCommands) {
	string ret;
	char output[30];

	ret = "Fire target: ";

	if (theCommands.bFire) {
		//+1 so side 1/group is 1
		sprintf(output, "%s Group %d", sides[theCommands.fireTarget.x].name.c_str(), theCommands.fireTarget.y + 1);
		ret+= output;
	} else
		ret+= "No target";

	return ret;
}

void CheckViewPos() {
	if (viewx > worldWidth - globalSettings.screenWidth)
		viewx = worldWidth - globalSettings.screenWidth;
	if (viewx < 0)
		viewx = 0;
	if (viewy > worldHeight - globalSettings.screenHeight)
		viewy = worldHeight - globalSettings.screenHeight;
	if (viewy < 0)
		viewy = 0;
}

void RestartPreBattle() {
	projectiles.clear();
	PreBattle::UnloadGraphics();

	for (int i = 0; i != sides.size(); ++i)
		sides[i].Reset();
}

} //end namespace

==> galaxyhack-1.62-src.tar.bz2: src/RTS.cpp <==
/*
   Copyright (C) 2004 by James Gregory
   Part of the GalaxyHack project
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License.
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY.
 
   See the COPYING file for more details.
*/

#include "RTS.h"
#include "Globals.h"
#include "Group.h"
#include "PreBattle.h"
#include "TerrainTile.h"
#include "Projectile.h"

#include <vector>
#include <list>

using std::vector;
using std::list;

namespace RTS {

RTS_State::RTS_State() {
	radarDragging = false;
	lastAITime = SDL_GetTicks();
	lastScrollTime = SDL_GetTicks();
}

RTS_State::~RTS_State() {
	projectiles.clear();

	if (gsTo != GST_PreBattle && gsTo != GST_Score)
		PreBattle::Unload();
	else if (gsTo == GST_PreBattle)
		RestartPreBattle();
}

void RTS_State::Main() {
	ScrollAndDrag();

	if (now - lastAITime > worldUpdateInterval && !(paused)) {
		RunGroupAI();
		RunMoveCommands();
		RunFireCommands();
		Upkeep();

		lastAITime = SDL_GetTicks();

		++frameCounter;
	}
	
	if (skipDisplayFrame || globalSettings.batch)
		return;
		
	DrawWorld();
	DrawAllWindows();
}

void RTS_State::MouseD(Uint8 button, Uint16 x, Uint16 y) {
	RTSMouseD(button, x, y);
}

void RTS_State::MouseU(Uint8 button, Uint16 x, Uint16 y) {
	RTSMouseU(button, x, y);
}

void RTS_State::MouseM(Uint8 state, Uint16 x, Uint16 y) {
	RTSMouseM(state, x, y);
}

void RTS_State::Keyboard(SDL_keysym& keysym) {
	RTSKeyboard(keysym);
}

void RTSMouseD(Uint8 button, Uint16 x, Uint16 y) {
	if (button == SDL_BUTTON_RIGHT)
		myWindows.push_back(GenWindow(x, y, RTS_BasePU, 0, 0, 0));
	
	//radar before groups
	if (button == SDL_BUTTON_LEFT
	&& x > radarRect.x
	&& x < radarRect.x + radarRect.w
	&& y > radarRect.y
	&& y < radarRect.y + radarRect.h) {
		radarDragging = true;
		RTSMouseM(0, x, y);
	} else {
		for (int i = 0; i != sides.size(); ++i) {
			for (int j = 0; j != sides[i].groups.size(); ++j)
				sides[i].groups[j].MouseD(button, x, y);
		}
	}
}

void RTSMouseU(Uint8 button, Uint16 x, Uint16 y) {
	if (button == SDL_BUTTON_LEFT)
		radarDragging = false;

	for (int i = 0; i != sides.size(); ++i) {
		for (int j = 0; j != sides[i].groups.size(); ++j)
			sides[i].groups[j].MouseU(button, x, y);
	}
}

void RTSMouseM(Uint8 state, Uint16 x, Uint16 y) {
	//radar
	if (radarDragging
		&& x > radarRect.x
		&& x < radarRect.x + radarRect.w
		&& y > radarRect.y
		&& y < radarRect.y + radarRect.h) {
			int tempCenterx = (x - radarRect.x) * worldWidth / radarRect.w;
			int tempCentery = (y - radarRect.y) * worldHeight / radarRect.h;

			viewx = tempCenterx - (globalSettings.screenWidth >> 1);
			viewy = tempCentery - (globalSettings.screenHeight >> 1);

			viewSide = -1;
	}
}

void RTSKeyboard(SDL_keysym& keysym) {
	switch(keysym.sym) {
	case SDLK_ESCAPE:
		if (gsCurrent == GST_PreBattle && PreBattle::pbState == PBS_Position)
			gsTo = GST_ForceSelect;
		else
			gsTo = GST_MainMenu;
		return;
		break;

	case SDLK_F1:
		MessageWindows(RTS_FollowView, 0, 0, all, none);
		break;

	case SDLK_F2:
		for (int i = 0; i != sides.size(); ++i) {
			for (int j = 0; j != sides[i].groups.size(); ++j)
				sides[i].groups[j].ToggleDrawNumber();
		}
		break;

	case SDLK_F3:
		for (int i = 0; i != sides.size(); ++i) {
			for (int j = 0; j != sides[i].groups.size(); ++j)
				sides[i].groups[j].ToggleDrawBound();
		}
		break;

	case SDLK_F11:
		myWindows.push_back(GenWindow(0, 0, RTS_RestartQ, 0, 0, 0));
		break;
	
	case SDLK_SPACE: {
		static bool speedSlider = false;
		static int sliderWinID = 0;		
		if (speedSlider) {
			try {
				Slider* pSlider = dynamic_cast<Slider*>(LocateWindow(sliderWinID));
				pSlider->WinMessage(WC_YouClose, 0, 0, sliderWinID, none);
				speedSlider = false;
				break;
			} catch (runtime_error e) {
			}
		}
		//if we fell through due to a runtime error because window was already closed, create a new one
		myWindows.push_back(GenWindow(topRightBoxRect.x, topRightBoxRect.y + topRightBoxRect.h, RTS_SetGameSpeed, 0, 0, 0));
		sliderWinID = windowIDs;
		speedSlider = true;
		}
		break;
	
	case SDLK_KP_PLUS:
		SetWorldUpdateInterval(worldUpdateInterval - 10);
		break;
		
	case SDLK_KP_MINUS:
		SetWorldUpdateInterval(worldUpdateInterval + 10);
		break;
	
	case SDLK_p:
		if (paused == false)
			paused = true;
		else if (worldUpdateInterval < maxWorldUpdateInterval)
			paused = false;
		break;
		
	case SDLK_i:
		SetWorldUpdateInterval(standardInterval);
		break;
		
	case SDLK_o:
		SetWorldUpdateInterval(0);
		break;
	}
}

void ScrollAndDrag() {
	int state, x, y;
	state = SDL_GetMouseState(&x, &y);

	//scrolling via mouse, also used to break out of follow view
	if (now - lastScrollTime > scrollInterval) {
		if (x > globalSettings.screenWidth - 8 || JSDL.keyboard[SDLK_RIGHT]) {
			viewx += screenMoveSpeed;
			viewSide = -1;
		}
		if (x < 8  || JSDL.keyboard[SDLK_LEFT]) {
			viewx -= screenMoveSpeed;
			viewSide = -1;
		}
		if (y > globalSettings.screenHeight - 8 || JSDL.keyboard[SDLK_DOWN]) {
			viewy += screenMoveSpeed;
			viewSide = -1;
		}
		if (y < 8 || JSDL.keyboard[SDLK_UP]) {
			viewy -= screenMoveSpeed;
			viewSide = -1;
		}

		lastScrollTime = SDL_GetTicks();
	}

	if (gsCurrent == GST_PreBattle) {
		for (int i = 0; i != sides.size(); ++i) {
			for (int j = 0; j != sides[i].groups.size(); ++j)
				sides[i].groups[j].Drag(state, x, y);
		}
	}
}

void RunGroupAI() {
	for (int i  = 0; i != sides.size(); ++i) {
		for (int j = 0; j != sides[i].groups.size(); ++j)
			sides[i].groups[j].RunGroupAI();
	}
}

void RunMoveCommands() {
	for (int i  = 0; i != sides.size(); ++i) {
		for (int j = 0; j != sides[i].groups.size(); ++j)
			sides[i].groups[j].Move();
	}

	for (list<Projectile>::iterator iter = projectiles.begin(); iter != projectiles.end();) {
		if (!iter->Move())
			iter = projectiles.erase(iter);
		else
			++iter;
	}
}

void RunFireCommands() {
	for (int i  = 0; i != sides.size(); ++i) {
		for (int j = 0; j != sides[i].groups.size(); ++j)
			sides[i].groups[j].RunFireCommands();
	}
}

void Upkeep() {
	for (int i  = 0; i != sides.size(); ++i) {
		for (int j = 0; j != sides[i].groups.size(); ++j)
			sides[i].groups[j].Upkeep();
	}

	int sidesLeft = 0;

	for (int i  = 0; i != sides.size(); ++i) {
		int sideTotalCS = sides[i].GetTotalCapShips();
		if (sideTotalCS)
			++sidesLeft;
	}

	if (sidesLeft < 2)
		gsTo = GST_Score;
	else if (globalSettings.batch && frameCounter > globalSettings.maxFrames)
		gsTo = GST_Score;
}

void DrawWorld() {
	FollowViewCenter();
	CheckViewPos();

	JSDL.BltFill(screenRect, 0);

	for (int i = 0; i != terrainTiles.size(); ++i)
		terrainTiles[i].DrawSelf();

	for (int i = 0; i != sides.size(); ++i) {
		for (int j = 0; j != sides[i].groups.size(); ++j)
			sides[i].groups[j].SetUSRect();
	}

	for (int i = 0; i != sides.size(); ++i) {
		for (int j = 0; j != sides[i].groups.size(); ++j)
			sides[i].groups[j].DrawSelfBackBack();
	}

	for (int i = 0; i != sides.size(); ++i) {
		for (int j = 0; j != sides[i].groups.size(); ++j)
			sides[i].groups[j].DrawSelfBack();
	}

	for (int i = 0; i != sides.size(); ++i) {
		for (int j = 0; j != sides[i].groups.size(); ++j)
			sides[i].groups[j].DrawSelfBack();
	}

	for (int i = 0; i != sides.size(); ++i) {
		for (int j = 0; j != sides[i].groups.size(); ++j)
			sides[i].groups[j].DrawSelfMiddle();
	}

	for (int i = 0; i != sides.size(); ++i) {
		for (int j = 0; j != sides[i].groups.size(); ++j)
			sides[i].groups[j].DrawSelfFront();
	}

	JSDL.LockBack();
	for (list<Projectile>::iterator iter = projectiles.begin(); iter != projectiles.end(); ++iter)
		iter->DrawSelfPixels();

	for (int i = 0; i != sides.size(); ++i) {
		for (int j = 0; j != sides[i].groups.size(); ++j)
			sides[i].groups[j].DrawSelfPixels();
	}
	JSDL.UnlockBack();

	for (list<Projectile>::iterator iter = projectiles.begin(); iter != projectiles.end(); ++iter)
		iter->DrawSelfBitmap();

	if (gsCurrent == GST_PreBattle) {
		for (int i = 0; i != sides.size(); ++i) {
			SDL_Rect tempRect;
			tempRect.x = sides[i].startingRect.x - viewx;
			tempRect.y = sides[i].startingRect.y - viewy;
			tempRect.w = sides[i].startingRect.w;
			tempRect.h = sides[i].startingRect.h;
			DrawRectBorder(tempRect, sides[i].color);
		}
	}

	DrawRadar();
}

void DrawRadar() {
	JSDL.BltFill(topRightBoxRect, gold);

	SDL_FillRect(radarSurface, 0, black);

	//Uint16 strided lpitch
	static const Uint16 lPitch = radarSurface->pitch >> 1;

	SDL_LockSurface(radarSurface);

	Uint16* videoBuffer = reinterpret_cast<Uint16*>(radarSurface->pixels);

	for (int i  = 0; i != sides.size(); ++i) {
		for (int j = 0; j != sides[i].groups.size(); ++j) {
			if (sides[i].groups[j].GetAlive()) {
				CoordsInt center = sides[i].groups[j].GetCenter();

				int radarx = center.x * radarRect.w / worldWidth;
				int radary = center.y * radarRect.h / worldHeight;

				videoBuffer[radary*lPitch + radarx] = sides[i].radarColor;
			}
		}
	}
	
	int radarx = viewx * radarRect.w / worldWidth;
	int radary = viewy * radarRect.h / worldHeight;
	
	static const int lineWidth = radarRect.w * globalSettings.screenWidth / worldWidth;
	static const int lineHeight = radarRect.h * globalSettings.screenHeight / worldHeight;
	
	for (int x = radarx; x != radarx + lineWidth + 1; ++x)
		videoBuffer[radary*lPitch + x] = white;
		
	for (int x = radarx; x != radarx + lineWidth + 1; ++x)
		videoBuffer[(radary + lineHeight)*lPitch + x] = white;
		
	for (int y = radary; y != radary + lineHeight + 1; ++y)
		videoBuffer[y*lPitch + radarx] = white;
		
	for (int y = radary; y != radary + lineHeight + 1; ++y)
		videoBuffer[y*lPitch + radarx + lineWidth] = white;

	SDL_UnlockSurface(radarSurface);

	JSDL.Blt(radarSurface, radarRect);
}

void SaveGame() {
    /* this should be staged so as to happen at the very end of a game loop, so we don't
    save some stuff out of sync with other stuff
    
    combine first 5 letters of each fleet fighting, divided by _s, to make a filename.
    if such a file already exists in saves directory, add a number on the end to make it unique
    
    Save all global variables - including static members, including random number "next"
    
    Save all variables for each side, and each group on each side, and each unit in each group
    */
}

void SetWorldUpdateInterval(int newValue) {
	worldUpdateInterval = newValue;
	
	if (worldUpdateInterval < 0)
		worldUpdateInterval = 0;
		
	if (worldUpdateInterval == maxWorldUpdateInterval)
		paused = true;
	else
		paused = false;
		
	if (worldUpdateInterval > maxWorldUpdateInterval)
		worldUpdateInterval = maxWorldUpdateInterval;

	if (worldUpdateInterval > standardInterval - 10 && worldUpdateInterval < standardInterval + 10)
		worldUpdateInterval = standardInterval;
}

//////

string MoveComToString(const AICommands& theCommands) {
	string ret;
	char output[30];

	if (!theCommands.bInverse)
		ret = "Moving towards ";
	else
		ret = "Moving away from ";

	switch (theCommands.moveCommand) {
	case MC_NoMove:
		ret = "Not moving";
		break;

	case MC_MoveCompass:
		switch (theCommands.moveTarget.y) {
		case 0:
			ret += "North";
			break;

		case 1:
			ret += "North-East";
			break;

		case 2:
			ret += "East";
			break;

		case 3:
			ret += "South-East";
			break;

		case 4:
			ret += "South";
			break;

		case 5:
			ret += "South-West";
			break;

		case 6:
			ret += "West";
			break;

		case 7:
			ret += "North-West";
			break;
		}
		break;

	case MC_MoveGroup:
		//+1 so side 1/group is 1
		sprintf(output, "%s Group %d", sides[theCommands.moveTarget.x].name.c_str(), theCommands.moveTarget.y + 1);

		ret += output;
		break;

	case MC_Patrol:
		ret = "Patrolling ";
		sprintf(output, "%s Group %d", sides[theCommands.moveTarget.x].name.c_str(), theCommands.moveTarget.y + 1);

		ret += output;
		break;
	}

	return ret;
}

string FireComToString(const AICommands& theCommands) {
	string ret;
	char output[30];

	ret = "Fire target: ";

	if (theCommands.bFire) {
		//+1 so side 1/group is 1
		sprintf(output, "%s Group %d", sides[theCommands.fireTarget.x].name.c_str(), theCommands.fireTarget.y + 1);
		ret+= output;
	} else
		ret+= "No target";

	return ret;
}

void CheckViewPos() {
	if (viewx > worldWidth - globalSettings.screenWidth)
		viewx = worldWidth - globalSettings.screenWidth;
	if (viewx < 0)
		viewx = 0;
	if (viewy > worldHeight - globalSettings.screenHeight)
		viewy = worldHeight - globalSettings.screenHeight;
	if (viewy < 0)
		viewy = 0;
}

void RestartPreBattle() {
	projectiles.clear();
	PreBattle::UnloadGraphics();

	for (int i = 0; i != sides.size(); ++i)
		sides[i].Reset();
}

} //end namespace
